@@ -82,7 +82,6 @@ static guint backend_signals[LAST_SIGNAL] = { 0, };
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
   ClutterMainContext *clutter_context;
 
   clutter_context = _clutter_context_get_default ();
@@ -96,11 +95,18 @@ clutter_backend_dispose (GObject *gobject)
       clutter_context->events_queue = NULL;
     }
 
-  g_free (backend->priv->font_name);
+  G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
+}
 
+static void
+clutter_backend_finalize (GObject *gobject)
+{
+  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
+
+  g_free (backend->priv->font_name);
   clutter_backend_set_font_options (backend, NULL);
 
-  G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
+  G_OBJECT_CLASS (clutter_backend_parent_class)->finalize (gobject);
 }
 
 static gfloat
@@ -115,12 +121,18 @@ get_units_per_em (ClutterBackend       *backend,
 
   if (font_desc == NULL)
     {
-      const gchar *font_name = clutter_backend_get_font_name (backend);
+      ClutterSettings *settings;
+      gchar *font_name = NULL;
+
+      settings = clutter_settings_get_default ();
+      g_object_get (settings, "font-name", &font_name, NULL);
 
       if (G_LIKELY (font_name != NULL && *font_name != '\0'))
         {
           font_desc = pango_font_description_from_string (font_name);
           free_font_desc = TRUE;
+
+          g_free (font_name);
         }
     }
 
@@ -196,6 +208,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->dispose = clutter_backend_dispose;
+  gobject_class->finalize = clutter_backend_finalize;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterBackendPrivate));
 
@@ -408,6 +421,17 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
           clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
 
           _cogl_onscreen_clutter_backend_set_size (width, height);
+
+          /* Eventually we will have a separate CoglFramebuffer for
+           * each stage and each one will track private projection
+           * matrix and viewport state, but until then we need to make
+           * sure we update the projection and viewport whenever we
+           * switch between stages.
+           *
+           * This dirty mechanism will ensure they are asserted before
+           * the next paint... */
+          _clutter_stage_dirty_viewport (stage);
+          _clutter_stage_dirty_projection (stage);
         }
 
       /* FIXME: With a NULL stage and thus no active context it may make more
@@ -416,17 +440,6 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
        * potential issue of GL calls with no context)
        */
       current_context_stage = new_stage;
-
-      /* if the new stage has a different size than the previous one
-       * we need to update the viewport; we do it by simply setting the
-       * SYNC_MATRICES flag and letting the next redraw cycle take care
-       * of calling glViewport()
-       */
-      if (current_context_stage)
-        {
-          CLUTTER_SET_PRIVATE_FLAGS (current_context_stage,
-                                     CLUTTER_ACTOR_SYNC_MATRICES);
-        }
     }
   else
     CLUTTER_NOTE (MULTISTAGE, "Stage is the same");
@@ -511,6 +524,36 @@ _clutter_backend_get_units_per_em (ClutterBackend       *backend,
   return priv->units_per_em;
 }
 
+void
+_clutter_backend_copy_event_data (ClutterBackend     *backend,
+                                  const ClutterEvent *src,
+                                  ClutterEvent       *dest)
+{
+  ClutterBackendClass *klass;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_return_if_fail (src != NULL);
+  g_return_if_fail (dest != NULL);
+
+  klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->copy_event_data != NULL)
+    klass->copy_event_data (backend, src, dest);
+}
+
+void
+_clutter_backend_free_event_data (ClutterBackend *backend,
+                                  ClutterEvent   *event)
+{
+  ClutterBackendClass *klass;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_return_if_fail (event != NULL);
+
+  klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->free_event_data != NULL)
+    klass->free_event_data (backend, event);
+}
+
 /**
  * clutter_get_default_backend:
  *
