@@ -78,9 +78,13 @@ enum
   PROP_DIRECTION,
   PROP_CENTER_X,
   PROP_CENTER_Y,
-  PROP_CENTER_Z
+  PROP_CENTER_Z,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 typedef struct {
   gdouble angle;
 } RotateFrameClosure;
@@ -275,6 +279,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_START] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_START,
                                    pspec);
@@ -292,6 +297,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ANGLE_END] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_ANGLE_END,
                                    pspec);
@@ -309,6 +315,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                              CLUTTER_TYPE_ROTATE_AXIS,
                              CLUTTER_Z_AXIS,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_AXIS] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_AXIS,
                                    pspec);
@@ -326,6 +333,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                              CLUTTER_TYPE_ROTATE_DIRECTION,
                              CLUTTER_ROTATE_CW,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DIRECTION] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_DIRECTION,
                                    pspec);
@@ -343,6 +351,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_X] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_X,
                                    pspec);
@@ -360,6 +369,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_Y] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Y,
                                    pspec);
@@ -377,6 +387,7 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
                             -G_MAXINT, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_CENTER_Z] = pspec;
   g_object_class_install_property (gobject_class,
                                    PROP_CENTER_Z,
                                    pspec);
@@ -400,7 +411,7 @@ clutter_behaviour_rotate_init (ClutterBehaviourRotate *rotate)
 
 /**
  * clutter_behaviour_rotate_new:
- * @alpha: a #ClutterAlpha, or %NULL
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @axis: the rotation axis
  * @direction: the rotation direction
  * @angle_start: the starting angle in degrees, between 0 and 360.
@@ -412,6 +423,10 @@ clutter_behaviour_rotate_init (ClutterBehaviourRotate *rotate)
  * <0, 360), if angle_start == angle_end, the behaviour will carry out a
  * single rotation of 360 degrees.
  *
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
  * Return value: the newly created #ClutterBehaviourRotate.
  *
  * Since: 0.4
@@ -475,7 +490,7 @@ clutter_behaviour_rotate_set_axis (ClutterBehaviourRotate *rotate,
     {
       priv->axis = axis;
 
-      g_object_notify (G_OBJECT (rotate), "axis");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_AXIS]);
     }
 }
 
@@ -521,7 +536,7 @@ clutter_behaviour_rotate_set_direction (ClutterBehaviourRotate *rotate,
     {
       priv->direction = direction;
 
-      g_object_notify (G_OBJECT (rotate), "direction");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_DIRECTION]);
     }
 }
 
@@ -581,14 +596,14 @@ clutter_behaviour_rotate_set_bounds (ClutterBehaviourRotate *rotate,
     {
       priv->angle_start = clamp_angle (angle_start);
 
-      g_object_notify (G_OBJECT (rotate), "angle-start");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_START]);
     }
 
   if (priv->angle_end != angle_end)
     {
       priv->angle_end = clamp_angle (angle_end);
 
-      g_object_notify (G_OBJECT (rotate), "angle-end");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_ANGLE_END]);
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
@@ -623,19 +638,19 @@ clutter_behaviour_rotate_set_center (ClutterBehaviourRotate *rotate,
   if (priv->center_x != x)
     {
       priv->center_x = x;
-      g_object_notify (G_OBJECT (rotate), "center-x");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_X]);
     }
 
   if (priv->center_y != y)
     {
       priv->center_y = y;
-      g_object_notify (G_OBJECT (rotate), "center-y");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Y]);
     }
 
   if (priv->center_z != z)
     {
       priv->center_z = z;
-      g_object_notify (G_OBJECT (rotate), "center-z");
+      _clutter_notify_by_pspec (G_OBJECT (rotate), obj_props[PROP_CENTER_Z]);
     }
 
   g_object_thaw_notify (G_OBJECT (rotate));
@@ -644,9 +659,9 @@ clutter_behaviour_rotate_set_center (ClutterBehaviourRotate *rotate,
 /**
  * clutter_behaviour_rotate_get_center:
  * @rotate: a #ClutterBehaviourRotate
- * @x: return location for the X center of rotation
- * @y: return location for the Y center of rotation
- * @z: return location for the Z center of rotation
+ * @x: (out): return location for the X center of rotation
+ * @y: (out): return location for the Y center of rotation
+ * @z: (out): return location for the Z center of rotation
  *
  * Retrieves the center of rotation set using
  * clutter_behaviour_rotate_set_center().
