@@ -176,9 +176,13 @@ struct _ClutterBehaviourPrivate
 enum
 {
   PROP_0,
-  PROP_ALPHA
+  PROP_ALPHA,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum {
   APPLIED,
   REMOVED,
@@ -284,6 +288,7 @@ static void
 clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec *pspec;
 
   object_class->dispose      = clutter_behaviour_dispose;
   object_class->set_property = clutter_behaviour_set_property;
@@ -299,13 +304,13 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
    *
    * Since: 0.2
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ALPHA,
-                                   g_param_spec_object ("alpha",
-                                                        "Alpha",
-                                                        "Alpha Object to drive the behaviour",
-                                                        CLUTTER_TYPE_ALPHA,
-                                                        CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_object ("alpha",
+                               P_("Alpha"),
+                               P_("Alpha Object to drive the behaviour"),
+                               CLUTTER_TYPE_ALPHA,
+                               CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ALPHA] = pspec;
+  g_object_class_install_property (object_class, PROP_ALPHA, pspec);
 
   klass->alpha_notify = clutter_behaviour_alpha_notify_unimplemented;
 
@@ -506,7 +511,7 @@ clutter_behaviour_get_nth_actor (ClutterBehaviour *behave,
 /**
  * clutter_behaviour_actors_foreach:
  * @behave: a #ClutterBehaviour
- * @func: a function called for each actor
+ * @func: (scope call): a function called for each actor
  * @data: optional data to be passed to the function, or %NULL
  *
  * Calls @func for every actor driven by @behave.
@@ -639,7 +644,7 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
                     priv->alpha, priv->notify_id);
     }
 
-  g_object_notify (G_OBJECT (behave), "alpha");
+  _clutter_notify_by_pspec (G_OBJECT (behave), obj_props[PROP_ALPHA]);
 }
 
 /**
@@ -650,7 +655,7 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
  * for derived classes to use this in there alpha notify method but use 
  * #clutter_behaviour_actors_foreach as it avoids alot of needless allocations.
  *
- * Return value: (transfer container) (element-type ClutterActor): a list of
+ * Return value: (transfer container) (element-type Clutter.Actor): a list of
  *   actors. You should free the returned list with g_slist_free() when
  *   finished using it.
  * 
