@@ -381,43 +381,43 @@ clutter_box_child_class_init (ClutterBoxChildClass *klass)
   gobject_class->finalize = clutter_box_child_finalize;
 
   pspec = g_param_spec_boolean ("expand",
-                                "Expand",
-                                "Allocate extra space for the child",
+                                P_("Expand"),
+                                P_("Allocate extra space for the child"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_EXPAND, pspec);
 
   pspec = g_param_spec_boolean ("x-fill",
-                                "Horizontal Fill",
-                                "Whether the child should receive priority "
-                                "when the container is allocating spare space "
-                                "on the horizontal axis",
+                                P_("Horizontal Fill"),
+                                P_("Whether the child should receive priority "
+                                   "when the container is allocating spare space "
+                                   "on the horizontal axis"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_FILL, pspec);
 
   pspec = g_param_spec_boolean ("y-fill",
-                                "Vertical Fill",
-                                "Whether the child should receive priority "
-                                "when the container is allocating spare space "
-                                "on the vertical axis",
+                                P_("Vertical Fill"),
+                                P_("Whether the child should receive priority "
+                                   "when the container is allocating spare space "
+                                   "on the vertical axis"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_Y_FILL, pspec);
 
   pspec = g_param_spec_enum ("x-align",
-                             "Horizontal Alignment",
-                             "Horizontal alignment of the actor within "
-                             "the cell",
+                             P_("Horizontal Alignment"),
+                             P_("Horizontal alignment of the actor within "
+                                "the cell"),
                              CLUTTER_TYPE_BOX_ALIGNMENT,
                              CLUTTER_BOX_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_ALIGN, pspec);
 
   pspec = g_param_spec_enum ("y-align",
-                             "Vertical Alignment",
-                             "Vertical alignment of the actor within "
-                             "the cell",
+                             P_("Vertical Alignment"),
+                             P_("Vertical alignment of the actor within "
+                                "the cell"),
                              CLUTTER_TYPE_BOX_ALIGNMENT,
                              CLUTTER_BOX_ALIGNMENT_CENTER,
                              CLUTTER_PARAM_READWRITE);
@@ -437,106 +437,22 @@ clutter_box_child_init (ClutterBoxChild *self)
   self->last_allocation = NULL;
 }
 
-static inline void
-allocate_fill (ClutterActor    *child,
-               ClutterActorBox *childbox,
-               ClutterBoxChild *box_child)
+static gdouble
+get_box_alignment_factor (ClutterBoxAlignment alignment)
 {
-  gfloat natural_width, natural_height;
-  gfloat min_width, min_height;
-  gfloat child_width, child_height;
-  gfloat available_width, available_height;
-  ClutterRequestMode request;
-  ClutterActorBox allocation = { 0, };
-  gdouble x_align, y_align;
-
-  if (box_child->x_align == CLUTTER_BOX_ALIGNMENT_START)
-    x_align = 0.0;
-  else if (box_child->x_align == CLUTTER_BOX_ALIGNMENT_CENTER)
-    x_align = 0.5;
-  else
-    x_align = 1.0;
-
-  if (box_child->y_align == CLUTTER_BOX_ALIGNMENT_START)
-    y_align = 0.0;
-  else if (box_child->y_align == CLUTTER_BOX_ALIGNMENT_CENTER)
-    y_align = 0.5;
-  else
-    y_align = 1.0;
-
-  available_width  = childbox->x2 - childbox->x1;
-  available_height = childbox->y2 - childbox->y1;
-
-  if (available_width < 0)
-    available_width = 0;
-
-  if (available_height < 0)
-    available_height = 0;
-
-  if (box_child->x_fill)
+  switch (alignment)
     {
-      allocation.x1 = childbox->x1;
-      allocation.x2 = ceilf (allocation.x1 + available_width);
+    case CLUTTER_BOX_ALIGNMENT_CENTER:
+      return 0.5;
+
+    case CLUTTER_BOX_ALIGNMENT_START:
+      return 0.0;
+
+    case CLUTTER_BOX_ALIGNMENT_END:
+      return 1.0;
     }
 
-  if (box_child->y_fill)
-    {
-      allocation.y1 = childbox->y1;
-      allocation.y2 = ceilf (allocation.y1 + available_height);
-    }
-
-  /* if we are filling horizontally and vertically then we're done */
-  if (box_child->x_fill && box_child->y_fill)
-    {
-      *childbox = allocation;
-      return;
-    }
-
-  request = clutter_actor_get_request_mode (child);
-  if (request == CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
-    {
-      clutter_actor_get_preferred_width (child, available_height,
-                                         &min_width,
-                                         &natural_width);
-
-      child_width = CLAMP (natural_width, min_width, available_width);
-
-      clutter_actor_get_preferred_height (child, child_width,
-                                          &min_height,
-                                          &natural_height);
-
-      child_height = CLAMP (natural_height, min_height, available_height);
-    }
-  else
-    {
-      clutter_actor_get_preferred_height (child, available_width,
-                                          &min_height,
-                                          &natural_height);
-
-      child_height = CLAMP (natural_height, min_height, available_height);
-
-      clutter_actor_get_preferred_width (child, child_height,
-                                         &min_width,
-                                         &natural_width);
-
-      child_width = CLAMP (natural_width, min_width, available_width);
-    }
-
-  if (!box_child->x_fill)
-    {
-      allocation.x1 = ceilf (childbox->x1
-                    + ((available_width - child_width) * x_align));
-      allocation.x2 = ceilf (allocation.x1 + child_width);
-    }
-
-  if (!box_child->y_fill)
-    {
-      allocation.y1 = ceilf (childbox->y1
-                    + ((available_height - child_height) * y_align));
-      allocation.y2 = ceilf (allocation.y1 + child_height);
-    }
-
-  *childbox = allocation;
+  return 0.0;
 }
 
 static GType
@@ -550,6 +466,7 @@ clutter_box_layout_set_container (ClutterLayoutManager *layout,
                                   ClutterContainer     *container)
 {
   ClutterBoxLayoutPrivate *priv = CLUTTER_BOX_LAYOUT (layout)->priv;
+  ClutterLayoutManagerClass *parent_class;
 
   priv->container = container;
 
@@ -560,12 +477,15 @@ clutter_box_layout_set_container (ClutterLayoutManager *layout,
       /* we need to change the :request-mode of the container
        * to match the orientation
        */
-      request_mode = (priv->is_vertical)
-                   ? CLUTTER_REQUEST_WIDTH_FOR_HEIGHT
-                   : CLUTTER_REQUEST_HEIGHT_FOR_WIDTH;
+      request_mode = (priv->is_vertical
+                      ? CLUTTER_REQUEST_HEIGHT_FOR_WIDTH
+                      : CLUTTER_REQUEST_WIDTH_FOR_HEIGHT);
       clutter_actor_set_request_mode (CLUTTER_ACTOR (priv->container),
                                       request_mode);
     }
+
+  parent_class = CLUTTER_LAYOUT_MANAGER_CLASS (clutter_box_layout_parent_class);
+  parent_class->set_container (layout, container);
 }
 
 static void
@@ -649,7 +569,7 @@ static void
 get_preferred_height (ClutterBoxLayout *self,
                       ClutterContainer *container,
                       GList            *children,
-                      gfloat            for_height,
+                      gfloat            for_width,
                       gfloat           *min_height_p,
                       gfloat           *natural_height_p)
 {
@@ -688,7 +608,7 @@ get_preferred_height (ClutterBoxLayout *self,
 
       clutter_actor_get_preferred_height (child,
                                           (priv->is_vertical)
-                                            ? for_height
+                                            ? for_width
                                             : -1,
                                           &child_min,
                                           &child_nat);
@@ -750,6 +670,8 @@ allocate_box_child (ClutterBoxLayout       *self,
       clutter_actor_get_preferred_height (child, avail_width,
                                           NULL, &child_nat);
 
+      child_nat = MIN (child_nat, avail_height);
+
       child_box.y1 = floorf (*position + 0.5);
 
       if (priv->is_homogeneous)
@@ -767,6 +689,8 @@ allocate_box_child (ClutterBoxLayout       *self,
       clutter_actor_get_preferred_width (child, avail_height,
                                          NULL, &child_nat);
 
+      child_nat = MIN (child_nat, avail_width);
+
       child_box.x1 = floorf (*position + 0.5);
 
       if (priv->is_homogeneous)
@@ -780,7 +704,17 @@ allocate_box_child (ClutterBoxLayout       *self,
       child_box.y2 = floorf (avail_height + 0.5);
     }
 
-  allocate_fill (child, &child_box, box_child);
+  clutter_actor_allocate_align_fill (child, &child_box,
+                                     get_box_alignment_factor (box_child->x_align),
+                                     get_box_alignment_factor (box_child->y_align),
+                                     box_child->x_fill,
+                                     box_child->y_fill,
+                                     flags);
+
+  /* retrieve the allocation computed and set by allocate_align_fill();
+   * since we call this *after* allocate(), it's just a cheap copy
+   */
+  clutter_actor_get_allocation_box (child, &child_box);
 
   if (priv->use_animations && priv->is_animating)
     {
@@ -883,7 +817,7 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
 {
   ClutterBoxLayoutPrivate *priv = CLUTTER_BOX_LAYOUT (layout)->priv;
   gfloat avail_width, avail_height, pref_width, pref_height;
-  gint n_expand_children, extra_space;
+  gint n_expand_children, n_children, extra_space;
   GList *children, *l;
   gfloat position;
   gboolean is_rtl;
@@ -916,7 +850,7 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
     }
 
   /* count the number of children with expand set to TRUE */
-  n_expand_children = 0;
+  n_children = n_expand_children = 0;
   for (l = children; l; l = l->next)
     {
       ClutterLayoutMeta *meta;
@@ -927,21 +861,18 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
 
       if (CLUTTER_BOX_CHILD (meta)->expand)
         n_expand_children++;
+
+      n_children++;
     }
 
   if (priv->is_homogeneous)
     {
-      gint n_children;
-
-      n_children = g_list_length (children);
       if (priv->is_vertical)
-	{
-	  extra_space = (avail_height - (n_children - 1)*priv->spacing)/n_children;
-	}
+	extra_space = (avail_height - (n_children - 1) * priv->spacing)
+                    / n_children;
       else
-	{
-	  extra_space = (avail_width - (n_children - 1)*priv->spacing)/n_children;
-	}
+	extra_space = (avail_width - (n_children - 1) * priv->spacing)
+                    / n_children;
     }
   else if (n_expand_children == 0)
     {
@@ -1156,9 +1087,9 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_boolean ("vertical",
-                                "Vertical",
-                                "Whether the layout should be vertical, rather"
-                                "than horizontal",
+                                P_("Vertical"),
+                                P_("Whether the layout should be vertical, rather"
+                                   "than horizontal"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_VERTICAL, pspec);
@@ -1172,9 +1103,9 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_boolean ("homogeneous",
-                                "Homogeneous",
-                                "Whether the layout should be homogeneous, i.e."
-                                "all childs get the same size",
+                                P_("Homogeneous"),
+                                P_("Whether the layout should be homogeneous, i.e."
+                                   "all childs get the same size"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_HOMOGENEOUS, pspec);
@@ -1188,8 +1119,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_boolean ("pack-start",
-                                "Pack Start",
-                                "Whether to pack items at the start of the box",
+                                P_("Pack Start"),
+                                P_("Whether to pack items at the start of the box"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_PACK_START, pspec);
@@ -1202,8 +1133,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_uint ("spacing",
-                             "Spacing",
-                             "Spacing between children",
+                             P_("Spacing"),
+                             P_("Spacing between children"),
                              0, G_MAXUINT, 0,
                              CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_SPACING, pspec);
@@ -1217,8 +1148,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_boolean ("use-animations",
-                                "Use Animations",
-                                "Whether layout changes should be animated",
+                                P_("Use Animations"),
+                                P_("Whether layout changes should be animated"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_USE_ANIMATIONS, pspec);
@@ -1239,8 +1170,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_ulong ("easing-mode",
-                              "Easing Mode",
-                              "The easing mode of the animations",
+                              P_("Easing Mode"),
+                              P_("The easing mode of the animations"),
                               0, G_MAXULONG,
                               CLUTTER_EASE_OUT_CUBIC,
                               CLUTTER_PARAM_READWRITE);
@@ -1257,8 +1188,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_uint ("easing-duration",
-                             "Easing Duration",
-                             "The duration of the animations",
+                             P_("Easing Duration"),
+                             P_("The duration of the animations"),
                              0, G_MAXUINT,
                              500,
                              CLUTTER_PARAM_READWRITE);
