@@ -366,6 +366,15 @@ clutter_cairo_texture_get_preferred_height (ClutterActor *actor,
     *natural_height = (gfloat) priv->height;
 }
 
+static gboolean
+clutter_cairo_texture_get_paint_volume (ClutterActor       *self,
+                                        ClutterPaintVolume *volume)
+{
+  return _clutter_actor_set_default_paint_volume (self,
+                                                  CLUTTER_TYPE_CAIRO_TEXTURE,
+                                                  volume);
+}
+
 static void
 clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
 {
@@ -378,6 +387,9 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
   gobject_class->get_property = clutter_cairo_texture_get_property;
   gobject_class->notify       = clutter_cairo_texture_notify;
 
+  actor_class->get_paint_volume =
+    clutter_cairo_texture_get_paint_volume;
+
   actor_class->get_preferred_width =
     clutter_cairo_texture_get_preferred_width;
   actor_class->get_preferred_height =
