@@ -179,7 +179,9 @@ static const GDebugKey clutter_pick_debug_keys[] = {
 static const GDebugKey clutter_paint_debug_keys[] = {
   { "disable-swap-events", CLUTTER_DEBUG_DISABLE_SWAP_EVENTS },
   { "disable-clipped-redraws", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS },
-  { "redraws", CLUTTER_DEBUG_REDRAWS }
+  { "redraws", CLUTTER_DEBUG_REDRAWS },
+  { "paint-volumes", CLUTTER_DEBUG_PAINT_VOLUMES },
+  { "disable-culling", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS }
 };
 
 #ifdef CLUTTER_ENABLE_PROFILE
@@ -224,47 +226,6 @@ clutter_get_accessibility_enabled (void)
   return cally_get_cally_initialized ();
 }
 
-
-void
-_clutter_stage_maybe_relayout (ClutterActor *stage)
-{
-  gfloat natural_width, natural_height;
-  ClutterActorBox box = { 0, };
-  CLUTTER_STATIC_TIMER (relayout_timer,
-                        "Mainloop", /* no parent */
-                        "Layouting",
-                        "The time spent reallocating the stage",
-                        0 /* no application private data */);
-
-  /* avoid reentrancy */
-  if (!CLUTTER_ACTOR_IN_RELAYOUT (stage))
-    {
-      CLUTTER_TIMER_START (_clutter_uprof_context, relayout_timer);
-      CLUTTER_NOTE (ACTOR, "Recomputing layout");
-
-      CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
-
-      natural_width = natural_height = 0;
-      clutter_actor_get_preferred_size (stage,
-                                        NULL, NULL,
-                                        &natural_width, &natural_height);
-
-      box.x1 = 0;
-      box.y1 = 0;
-      box.x2 = natural_width;
-      box.y2 = natural_height;
-
-      CLUTTER_NOTE (ACTOR, "Allocating (0, 0 - %d, %d) for the stage",
-                    (int) natural_width,
-                    (int) natural_height);
-
-      clutter_actor_allocate (stage, &box, CLUTTER_ALLOCATION_NONE);
-
-      CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
-      CLUTTER_TIMER_STOP (_clutter_uprof_context, relayout_timer);
-    }
-}
-
 void
 _clutter_do_redraw (ClutterStage *stage)
 {
@@ -274,12 +235,9 @@ _clutter_do_redraw (ClutterStage *stage)
 
   ctx = _clutter_context_get_default ();
 
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE);
   _clutter_stage_reset_picks_per_frame_counter (stage);
 
-  /* Before we can paint, we have to be sure we have the latest layout */
-  _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
-
   _clutter_backend_ensure_context (ctx->backend, stage);
 
   /* Setup FPS count - not currently across *all* stages rather than per */
@@ -597,7 +555,7 @@ _clutter_do_pick (ClutterStage   *stage,
   /* It's possible that we currently have a static scene and have renderered a
    * full, unclipped pick buffer. If so we can simply continue to read from
    * this cached buffer until the scene next changes. */
-  if (_clutter_stage_get_pick_buffer_valid (stage, mode))
+  if (_clutter_stage_get_pick_buffer_valid (stage))
     {
       CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
       cogl_read_pixels (x, y, 1, 1,
@@ -665,7 +623,7 @@ _clutter_do_pick (ClutterStage   *stage,
   */
   CLUTTER_TIMER_START (_clutter_uprof_context, pick_paint);
   context->pick_mode = mode;
-  clutter_actor_paint (CLUTTER_ACTOR (stage));
+  _clutter_stage_do_paint (stage, NULL);
   context->pick_mode = CLUTTER_PICK_NONE;
   CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_paint);
 
@@ -674,11 +632,9 @@ _clutter_do_pick (ClutterStage   *stage,
       if (G_LIKELY (!(clutter_pick_debug_flags &
                       CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
         cogl_clip_pop ();
-
-      _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
     }
   else
-    _clutter_stage_set_pick_buffer_valid (stage, TRUE, mode);
+    _clutter_stage_set_pick_buffer_valid (stage, TRUE);
 
   /* Make sure Cogl flushes any batched geometry to the GPU driver */
   cogl_flush ();
@@ -1612,6 +1568,17 @@ clutter_init_real (GError **error)
   if (!_clutter_backend_post_parse (backend, error))
     return CLUTTER_INIT_ERROR_BACKEND;
 
+  /* If we are displaying the regions that would get redrawn with clipped
+   * redraws enabled we actually have to disable the clipped redrawing
+   * because otherwise we end up with nasty trails of rectangles everywhere.
+   */
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)
+    clutter_paint_debug_flags |= CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS;
+
+  /* The same is true when drawing the outlines of paint volumes... */
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_PAINT_VOLUMES)
+    clutter_paint_debug_flags |= CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS;
+
   /* this will take care of initializing Cogl's state and
    * query the GL machinery for features
    */
