@@ -210,8 +210,6 @@
 #include "clutter-private.h"
 #include "clutter-debug.h"
 
-#include "json/json-parser.h"
-
 enum
 {
   PROP_0,
@@ -571,7 +569,7 @@ clutter_script_load_from_data (ClutterScript  *script,
  * Retrieves the object bound to @name. This function does not increment
  * the reference count of the returned object.
  *
- * Return value: (transfer none): the named object, or %NULL if no object
+ * Return value: : (transfer none): the named object, or %NULL if no object
  *   with the given name was available
  *
  * Since: 0.6
@@ -973,7 +971,7 @@ connect_each_object (gpointer key,
 }
 
 /**
- * clutter_script_connect_signals_full:
+ * clutter_script_connect_signals_full: (skip)
  * @script: a #ClutterScript
  * @func: signal connection function
  * @user_data: data to be passed to the signal handlers, or %NULL
