@@ -33,10 +33,11 @@
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_ACTOR_BOX  (clutter_actor_box_get_type ())
-#define CLUTTER_TYPE_GEOMETRY   (clutter_geometry_get_type ())
-#define CLUTTER_TYPE_KNOT       (clutter_knot_get_type ())
-#define CLUTTER_TYPE_VERTEX     (clutter_vertex_get_type ())
+#define CLUTTER_TYPE_ACTOR_BOX          (clutter_actor_box_get_type ())
+#define CLUTTER_TYPE_GEOMETRY           (clutter_geometry_get_type ())
+#define CLUTTER_TYPE_KNOT               (clutter_knot_get_type ())
+#define CLUTTER_TYPE_PAINT_VOLUME       (clutter_paint_volume_get_type ())
+#define CLUTTER_TYPE_VERTEX             (clutter_vertex_get_type ())
 
 /* Forward delarations to avoid header catch 22's */
 typedef struct _ClutterActor            ClutterActor;
@@ -91,6 +92,23 @@ typedef struct _ClutterGeometry         ClutterGeometry;
 typedef struct _ClutterKnot             ClutterKnot;
 typedef struct _ClutterVertex           ClutterVertex;
 
+/**
+ * ClutterPaintVolume:
+ *
+ * <structname>ClutterPaintVolume</structname> is an opaque structure
+ * whose members cannot be directly accessed.
+ *
+ * A <structname>ClutterPaintVolume</structname> represents an
+ * a bounding volume whos internal representation isn't defined but
+ * can be set and queried in terms of an axis aligned bounding box.
+ *
+ * Other internal representation and methods for describing the
+ * bounding volume may be added in the future.
+ *
+ * Since: 1.4
+ */
+typedef struct _ClutterPaintVolume      ClutterPaintVolume;
+
 /**
  * ClutterVertex:
  * @x: X coordinate of the vertex
@@ -167,6 +185,9 @@ void             clutter_actor_box_interpolate   (const ClutterActorBox *initial
                                                   gdouble                progress,
                                                   ClutterActorBox       *result);
 void             clutter_actor_box_clamp_to_pixel (ClutterActorBox       *box);
+void             clutter_actor_box_union          (const ClutterActorBox *a,
+                                                   const ClutterActorBox *b,
+                                                   ClutterActorBox       *result);
 
 /**
  * ClutterGeometry:
@@ -188,9 +209,11 @@ struct _ClutterGeometry
 
 GType clutter_geometry_get_type (void) G_GNUC_CONST;
 
-void  clutter_geometry_union (const ClutterGeometry *geometry_a,
-                              const ClutterGeometry *geometry_b,
-                              ClutterGeometry       *result);
+void      clutter_geometry_union      (const ClutterGeometry *geometry_a,
+                                       const ClutterGeometry *geometry_b,
+                                       ClutterGeometry       *result);
+gboolean  clutter_geometry_intersects (const ClutterGeometry *geometry0,
+                                       const ClutterGeometry *geometry1);
 
 /**
  * ClutterKnot:
@@ -397,7 +420,7 @@ typedef enum {
 typedef enum
 {
   CLUTTER_FONT_MIPMAPPING = (1 << 0),
-  CLUTTER_FONT_HINTING    = (1 << 1),
+  CLUTTER_FONT_HINTING    = (1 << 1)
 } ClutterFontFlags;
 
 /**
@@ -431,6 +454,30 @@ typedef enum {
   CLUTTER_FRAGMENT_SHADER
 } ClutterShaderType;
 
+GType clutter_paint_volume_get_type (void) G_GNUC_CONST;
+
+ClutterPaintVolume *clutter_paint_volume_copy                (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_free                (ClutterPaintVolume       *pv);
+
+void                clutter_paint_volume_set_origin          (ClutterPaintVolume       *pv,
+                                                              const ClutterVertex      *origin);
+void                clutter_paint_volume_get_origin          (const ClutterPaintVolume *pv,
+                                                              ClutterVertex            *vertex);
+void                clutter_paint_volume_set_width           (ClutterPaintVolume       *pv,
+                                                              gfloat                    width);
+gfloat              clutter_paint_volume_get_width           (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_set_height          (ClutterPaintVolume       *pv,
+                                                              gfloat                    height);
+gfloat              clutter_paint_volume_get_height          (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_set_depth           (ClutterPaintVolume       *pv,
+                                                              gfloat                    depth);
+gfloat              clutter_paint_volume_get_depth           (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_union               (ClutterPaintVolume       *pv,
+                                                              const ClutterPaintVolume *another_pv);
+
+gboolean            clutter_paint_volume_set_from_allocation (ClutterPaintVolume       *pv,
+                                                              ClutterActor             *actor);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TYPES_H__ */
