@@ -22,6 +22,7 @@
  *
  * Authors:
  *   Damien Lespiau <damien.lespiau@intel.com>
+ *   Robert Bragg <robert@linux.intel.com>
  */
 
 #ifndef __COGL_BUFFER_PRIVATE_H__
@@ -29,7 +30,8 @@
 
 #include <glib.h>
 
-#include "cogl-handle.h"
+#include "cogl.h"
+#include "cogl-object.h"
 #include "cogl-buffer.h"
 
 G_BEGIN_DECLS
@@ -49,15 +51,16 @@ typedef struct _CoglBufferVtable CoglBufferVtable;
 
 struct _CoglBufferVtable
 {
-  guint8 * (* map)          (CoglBuffer       *buffer,
-                             CoglBufferAccess  access);
+  guint8 * (* map) (CoglBuffer       *buffer,
+                    CoglBufferAccess  access,
+                    CoglBufferMapHint hints);
 
-  void     (* unmap)        (CoglBuffer *buffer);
+  void (* unmap) (CoglBuffer *buffer);
 
-  gboolean (* set_data)     (CoglBuffer   *buffer,
-                             unsigned int  offset,
-                             const guint8 *data,
-                             unsigned int  size);
+  gboolean (* set_data) (CoglBuffer   *buffer,
+                         unsigned int  offset,
+                         const guint8 *data,
+                         unsigned int  size);
 };
 
 typedef enum _CoglBufferFlags
@@ -67,10 +70,26 @@ typedef enum _CoglBufferFlags
   COGL_BUFFER_FLAG_MAPPED        = 1UL << 1
 } CoglBufferFlags;
 
+typedef enum {
+  COGL_BUFFER_USAGE_HINT_TEXTURE,
+  COGL_BUFFER_USAGE_HINT_VERTICES
+} CoglBufferUsageHint;
+
+typedef enum {
+  COGL_BUFFER_BIND_TARGET_PIXEL_PACK,
+  COGL_BUFFER_BIND_TARGET_PIXEL_UNPACK,
+  COGL_BUFFER_BIND_TARGET_VERTEX_ARRAY,
+  COGL_BUFFER_BIND_TARGET_VERTEX_INDICES_ARRAY,
+
+  COGL_BUFFER_BIND_TARGET_COUNT
+} CoglBufferBindTarget;
+
 struct _CoglBuffer
 {
-  CoglHandleObject        _parent;
-  const CoglBufferVtable *vtable;
+  CoglObject              _parent;
+  CoglBufferVtable        vtable;
+
+  CoglBufferBindTarget    last_target;
 
   CoglBufferFlags         flags;
 
@@ -85,16 +104,44 @@ struct _CoglBuffer
                                       * fallback paths */
 };
 
-void    _cogl_buffer_initialize         (CoglBuffer          *buffer,
-                                         unsigned int         size,
-                                         CoglBufferUsageHint  usage_hint,
-                                         CoglBufferUpdateHint update_hint);
-void    _cogl_buffer_fini               (CoglBuffer *buffer);
-void    _cogl_buffer_bind               (CoglBuffer *buffer,
-                                         GLenum      target);
-GLenum  _cogl_buffer_access_to_gl_enum  (CoglBufferAccess access);
-GLenum  _cogl_buffer_hints_to_gl_enum   (CoglBufferUsageHint  usage_hint,
-                                         CoglBufferUpdateHint update_hint);
+/* This is used to register a type to the list of handle types that
+   will be considered a texture in cogl_is_texture() */
+void
+_cogl_buffer_register_buffer_type (GQuark type);
+
+#define COGL_BUFFER_DEFINE(TypeName, type_name)                         \
+  COGL_OBJECT_DEFINE_WITH_CODE                                          \
+  (TypeName, type_name,                                                 \
+   _cogl_buffer_register_buffer_type (_cogl_object_                     \
+                                      ## type_name ## _get_type ()))
+
+void
+_cogl_buffer_initialize (CoglBuffer          *buffer,
+                         unsigned int         size,
+                         gboolean             use_malloc,
+                         CoglBufferBindTarget default_target,
+                         CoglBufferUsageHint  usage_hint,
+                         CoglBufferUpdateHint update_hint);
+
+void
+_cogl_buffer_fini (CoglBuffer *buffer);
+
+void
+_cogl_buffer_bind (CoglBuffer *buffer,
+                   CoglBufferBindTarget target);
+
+void
+_cogl_buffer_unbind (CoglBuffer *buffer);
+
+CoglBufferUsageHint
+_cogl_buffer_get_usage_hint (CoglBuffer *buffer);
+
+GLenum
+_cogl_buffer_access_to_gl_enum (CoglBufferAccess access);
+
+GLenum
+_cogl_buffer_hints_to_gl_enum (CoglBufferUsageHint  usage_hint,
+                               CoglBufferUpdateHint update_hint);
 
 G_END_DECLS
 
