@@ -16,12 +16,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authors:
  *   Damien Lespiau <damien.lespiau@intel.com>
+ *   Robert Bragg <robert@linux.intel.com>
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -45,11 +46,28 @@ G_BEGIN_DECLS
  * OpenGL implementation allows it, COGL will use Pixel Buffer Objects.
  */
 
+/* All of the cogl-buffer API is currently experimental so we suffix
+ * the actual symbols with _EXP so if somone is monitoring for ABI
+ * changes it will hopefully be clearer to them what's going on if any
+ * of the symbols dissapear at a later date.
+ */
+#define cogl_is_buffer  cogl_is_buffer_EXP
+#define cogl_buffer_get_size cogl_buffer_get_size_EXP
+#define cogl_buffer_set_usage_hint cogl_buffer_set_usage_hint_EXP
+#define cogl_buffer_get_usage_hint cogl_buffer_get_usage_hint_EXP
+#define cogl_buffer_set_update_hint cogl_buffer_set_update_hint_EXP
+#define cogl_buffer_get_update_hint cogl_buffer_get_update_hint_EXP
+#define cogl_buffer_map cogl_buffer_map_EXP
+#define cogl_buffer_unmap cogl_buffer_unmap_EXP
+#define cogl_buffer_set_data cogl_buffer_set_data_EXP
+
+typedef struct _CoglBuffer CoglBuffer;
+
 /**
  * cogl_is_buffer:
- * @handle: a #CoglHandle to test
+ * @buffer: a buffer object
  *
- * Checks whether @handle is a buffer handle.
+ * Checks whether @buffer is a buffer object.
  *
  * Return value: %TRUE if the handle is a CoglBuffer, and %FALSE otherwise
  *
@@ -57,11 +75,11 @@ G_BEGIN_DECLS
  * Stability: Unstable
  */
 gboolean
-cogl_is_buffer (CoglHandle handle);
+cogl_is_buffer (const void *object);
 
 /**
  * cogl_buffer_get_size:
- * @handle: a buffer handle
+ * @buffer: a buffer object
  *
  * Retrieves the size of buffer
  *
@@ -71,51 +89,7 @@ cogl_is_buffer (CoglHandle handle);
  * Stability: Unstable
  */
 unsigned int
-cogl_buffer_get_size (CoglHandle handle);
-
-/**
- * CoglBufferUsageHint:
- * @COGL_BUFFER_USAGE_HINT_TEXTURE: the buffer will be used as a source data
- *   for a texture
- *
- * The usage hint on a buffer allows the user to give some clue on how the
- * buffer will be used.
- *
- * Since: 1.2
- * Stability: Unstable
- */
-typedef enum { /*< prefix=COGL_BUFFER_USAGE_HINT >*/
-  COGL_BUFFER_USAGE_HINT_TEXTURE,
-} CoglBufferUsageHint;
-
-/**
- * cogl_buffer_set_usage_hint:
- * @handle: a buffer handle
- * @hint: the new hint
- *
- * Sets the usage hint on a buffer. See #CoglBufferUsageHint for a description
- * of the available hints.
- *
- * Since: 1.2
- * Stability: Unstable
- */
-void
-cogl_buffer_set_usage_hint (CoglHandle          handle,
-                            CoglBufferUsageHint hint);
-
-/**
- * cogl_buffer_get_usage_hint:
- * @handle: a buffer handle
- *
- * Retrieves the usage hint set using cogl_buffer_set_usage_hint()
- *
- * Return value: the #CoglBufferUsageHint currently used by the buffer
- *
- * Since: 1.2
- * Stability: Unstable
- */
-CoglBufferUsageHint
-cogl_buffer_get_usage_hint (CoglHandle handle);
+cogl_buffer_get_size (CoglBuffer *buffer);
 
 /**
  * CoglBufferUpdateHint:
@@ -138,7 +112,7 @@ typedef enum { /*< prefix=COGL_BUFFER_UPDATE_HINT >*/
 
 /**
  * cogl_buffer_set_update_hint:
- * @handle: a buffer handle
+ * @buffer: a buffer object
  * @hint: the new hint
  *
  * Sets the update hint on a buffer. See #CoglBufferUpdateHint for a description
@@ -148,12 +122,12 @@ typedef enum { /*< prefix=COGL_BUFFER_UPDATE_HINT >*/
  * Stability: Unstable
  */
 void
-cogl_buffer_set_update_hint (CoglHandle           handle,
+cogl_buffer_set_update_hint (CoglBuffer          *buffer,
                              CoglBufferUpdateHint hint);
 
 /**
  * cogl_buffer_get_update_hint:
- * @handle: a buffer handle
+ * @buffer: a buffer object
  *
  * Retrieves the update hints set using cogl_buffer_set_update_hint()
  *
@@ -163,7 +137,7 @@ cogl_buffer_set_update_hint (CoglHandle           handle,
  * Stability: Unstable
  */
 CoglBufferUpdateHint
-cogl_buffer_get_update_hint (CoglHandle handle);
+cogl_buffer_get_update_hint (CoglBuffer *buffer);
 
 /**
  * CoglBufferAccess:
@@ -180,29 +154,58 @@ cogl_buffer_get_update_hint (CoglHandle handle);
 typedef enum { /*< prefix=COGL_BUFFER_ACCESS >*/
  COGL_BUFFER_ACCESS_READ       = 1 << 0,
  COGL_BUFFER_ACCESS_WRITE      = 1 << 1,
- COGL_BUFFER_ACCESS_READ_WRITE = COGL_BUFFER_ACCESS_READ |
-                                 COGL_BUFFER_ACCESS_WRITE
+ COGL_BUFFER_ACCESS_READ_WRITE = COGL_BUFFER_ACCESS_READ | COGL_BUFFER_ACCESS_WRITE
 } CoglBufferAccess;
 
+
+/**
+ * CoglBufferMapHint:
+ * COGL_BUFFER_MAP_HINT_DISCARD: Tells Cogl that you plan to replace
+ *                               all the buffer's contents.
+ *
+ * Hints to Cogl about how you are planning to modify the data once it
+ * is mapped.
+ *
+ * Since: 1.4
+ * Stability: Unstable
+ */
+typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
+    COGL_BUFFER_MAP_HINT_DISCARD = 1 << 0
+} CoglBufferMapHint;
+
 /**
  * cogl_buffer_map:
- * @handle: a buffer handle
- * @access: how the mapped buffer will by use by the application
+ * @buffer: a buffer object
+ * @access: how the mapped buffer will be used by the application
+ * @hints: A mask of #CoglBufferMapHint<!-- -->s that tell Cogl how
+ *        the data will be modified once mapped.
  *
  * Maps the buffer into the application address space for direct access.
  *
+ * It is strongly recommended that you pass
+ * %COGL_BUFFER_MAP_HINT_DISCARD as a hint if you are going to replace
+ * all the buffer's data. This way if the buffer is currently being
+ * used by the GPU then the driver won't have to stall the CPU and
+ * wait for the hardware to finish because it can instead allocate a
+ * new buffer to map.
+ *
+ * The behaviour is undefined if you access the buffer in a way
+ * conflicting with the @access mask you pass. It is also an error to
+ * release your last reference while the buffer is mapped.
+ *
  * Return value: A pointer to the mapped memory or %NULL is the call fails
  *
  * Since: 1.2
  * Stability: Unstable
  */
 guint8 *
-cogl_buffer_map (CoglHandle       handle,
-                 CoglBufferAccess access);
+cogl_buffer_map (CoglBuffer        *buffer,
+                 CoglBufferAccess   access,
+                 CoglBufferMapHint  hints);
 
 /**
  * cogl_buffer_unmap:
- * @handle: a buffer handle
+ * @buffer: a buffer object
  *
  * Unmaps a buffer previously mapped by cogl_buffer_map().
  *
@@ -210,11 +213,11 @@ cogl_buffer_map (CoglHandle       handle,
  * Stability: Unstable
  */
 void
-cogl_buffer_unmap (CoglHandle handle);
+cogl_buffer_unmap (CoglBuffer *buffer);
 
 /**
  * cogl_buffer_set_data:
- * @handle: a buffer handle
+ * @buffer: a buffer object
  * @offset: destination offset (in bytes) in the buffer
  * @data: a pointer to the data to be copied into the buffer
  * @size: number of bytes to copy
@@ -229,59 +232,11 @@ cogl_buffer_unmap (CoglHandle handle);
  * Stability: Unstable
  */
 gboolean
-cogl_buffer_set_data (CoglHandle    handle,
+cogl_buffer_set_data (CoglBuffer  *buffer,
                       gsize        offset,
                       const guint8 *data,
                       gsize        size);
 
-/* the functions above are experimental, the actual symbols are suffixed by
- * _EXP so we can ensure ABI compatibility and leave the cogl_buffer namespace
- * free for future use. A bunch of defines translates the symbols documented
- * above into the real symbols */
-
-gboolean
-cogl_is_buffer_EXP (CoglHandle handle);
-
-unsigned int
-cogl_buffer_get_size_EXP (CoglHandle handle);
-
-void
-cogl_buffer_set_usage_hint_EXP (CoglHandle          handle,
-                                CoglBufferUsageHint hint);
-
-CoglBufferUsageHint
-cogl_buffer_get_usage_hint_EXP (CoglHandle handle);
-
-void
-cogl_buffer_set_update_hint_EXP (CoglHandle           handle,
-                                 CoglBufferUpdateHint hint);
-
-CoglBufferUpdateHint
-cogl_buffer_get_update_hint_EXP (CoglHandle handle);
-
-guint8 *
-cogl_buffer_map_EXP (CoglHandle       handle,
-                     CoglBufferAccess access);
-
-void
-cogl_buffer_unmap_EXP (CoglHandle handle);
-
-gboolean
-cogl_buffer_set_data_EXP (CoglHandle    handle,
-                          gsize        offset,
-                          const guint8 *data,
-                          gsize        size);
-
-#define cogl_is_buffer  cogl_is_buffer_EXP
-#define cogl_buffer_get_size cogl_buffer_get_size_EXP
-#define cogl_buffer_set_usage_hint cogl_buffer_set_usage_hint_EXP
-#define cogl_buffer_get_usage_hint cogl_buffer_get_usage_hint_EXP
-#define cogl_buffer_set_update_hint cogl_buffer_set_update_hint_EXP
-#define cogl_buffer_get_update_hint cogl_buffer_get_update_hint_EXP
-#define cogl_buffer_map cogl_buffer_map_EXP
-#define cogl_buffer_unmap cogl_buffer_unmap_EXP
-#define cogl_buffer_set_data cogl_buffer_set_data_EXP
-
 G_END_DECLS
 
 #endif /* __COGL_BUFFER_H__ */
