@@ -42,7 +42,7 @@
 
 static void _cogl_sub_texture_free (CoglSubTexture *sub_tex);
 
-COGL_HANDLE_DEFINE (SubTexture, sub_texture);
+COGL_TEXTURE_INTERNAL_DEFINE (SubTexture, sub_texture);
 
 static const CoglTextureVtable cogl_sub_texture_vtable;
 
@@ -224,12 +224,17 @@ _cogl_sub_texture_foreach_sub_texture_in_region (
 }
 
 static void
-_cogl_sub_texture_set_wrap_mode_parameter (CoglTexture *tex,
-                                           GLenum wrap_mode)
+_cogl_sub_texture_set_wrap_mode_parameters (CoglTexture *tex,
+                                            GLenum wrap_mode_s,
+                                            GLenum wrap_mode_t,
+                                            GLenum wrap_mode_p)
 {
   CoglSubTexture *sub_tex = COGL_SUB_TEXTURE (tex);
 
-  _cogl_texture_set_wrap_mode_parameter (sub_tex->full_texture, wrap_mode);
+  _cogl_texture_set_wrap_mode_parameters (sub_tex->full_texture,
+                                          wrap_mode_s,
+                                          wrap_mode_t,
+                                          wrap_mode_p);
 }
 
 static void
@@ -238,7 +243,8 @@ _cogl_sub_texture_free (CoglSubTexture *sub_tex)
   cogl_handle_unref (sub_tex->next_texture);
   cogl_handle_unref (sub_tex->full_texture);
 
-  g_free (sub_tex);
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (sub_tex));
 }
 
 CoglHandle
@@ -268,7 +274,7 @@ _cogl_sub_texture_new (CoglHandle next_texture,
   /* If the next texture is also a sub texture we can avoid one level
      of indirection by referencing the full texture of that texture
      instead. */
-  if (cogl_is_sub_texture (next_texture))
+  if (_cogl_is_sub_texture (next_texture))
     {
       CoglSubTexture *other_sub_tex =
         _cogl_sub_texture_pointer_from_handle (next_texture);
@@ -379,11 +385,12 @@ _cogl_sub_texture_set_filters (CoglTexture *tex,
 }
 
 static void
-_cogl_sub_texture_ensure_mipmaps (CoglTexture *tex)
+_cogl_sub_texture_pre_paint (CoglTexture *tex,
+                             CoglTexturePrePaintFlags flags)
 {
   CoglSubTexture *sub_tex = COGL_SUB_TEXTURE (tex);
 
-  _cogl_texture_ensure_mipmaps (sub_tex->full_texture);
+  _cogl_texture_pre_paint (sub_tex->full_texture, flags);
 }
 
 static void
@@ -399,23 +406,16 @@ _cogl_sub_texture_set_region (CoglTexture    *tex,
                               int             dst_y,
                               unsigned int    dst_width,
                               unsigned int    dst_height,
-                              int             width,
-                              int             height,
-                              CoglPixelFormat format,
-                              unsigned int    rowstride,
-                              const guint8   *data)
+                              CoglBitmap     *bmp)
 {
   CoglSubTexture  *sub_tex = COGL_SUB_TEXTURE (tex);
 
-  return cogl_texture_set_region (sub_tex->full_texture,
-                                  src_x, src_y,
-                                  dst_x + sub_tex->sub_x,
-                                  dst_y + sub_tex->sub_y,
-                                  dst_width, dst_height,
-                                  width, height,
-                                  format,
-                                  rowstride,
-                                  data);
+  return _cogl_texture_set_region_from_bitmap (sub_tex->full_texture,
+                                               src_x, src_y,
+                                               dst_x + sub_tex->sub_x,
+                                               dst_y + sub_tex->sub_y,
+                                               dst_width, dst_height,
+                                               bmp);
 }
 
 static void
@@ -441,7 +441,7 @@ _cogl_sub_texture_copy_region (guint8 *dst,
     }
 }
 
-static int
+static gboolean
 _cogl_sub_texture_get_data (CoglTexture     *tex,
                             CoglPixelFormat  format,
                             unsigned int     rowstride,
@@ -450,7 +450,7 @@ _cogl_sub_texture_get_data (CoglTexture     *tex,
   CoglSubTexture *sub_tex = COGL_SUB_TEXTURE (tex);
   unsigned int full_rowstride;
   guint8 *full_data;
-  int byte_size, full_size;
+  gboolean ret = TRUE;
   int bpp;
   int full_tex_width, full_tex_height;
 
@@ -459,30 +459,16 @@ _cogl_sub_texture_get_data (CoglTexture     *tex,
      texture_get_sub_data virtual and it can just munge the texture
      coordinates */
 
-  /* Default to internal format if none specified */
-  if (format == COGL_PIXEL_FORMAT_ANY)
-    format = cogl_texture_get_format (sub_tex->full_texture);
-
   full_tex_width = cogl_texture_get_width (sub_tex->full_texture);
   full_tex_height = cogl_texture_get_height (sub_tex->full_texture);
 
-  /* Rowstride from texture width if none specified */
   bpp = _cogl_get_format_bpp (format);
-  if (rowstride == 0)
-    rowstride = sub_tex->sub_width * bpp;
-
-  /* Return byte size if only that requested */
-  byte_size = sub_tex->sub_height * rowstride;
-  if (data == NULL)
-    return byte_size;
 
   full_rowstride = _cogl_get_format_bpp (format) * full_tex_width;
   full_data = g_malloc (full_rowstride * full_tex_height);
 
-  full_size = cogl_texture_get_data (sub_tex->full_texture, format,
-                                     full_rowstride, full_data);
-
-  if (full_size)
+  if (cogl_texture_get_data (sub_tex->full_texture, format,
+                             full_rowstride, full_data))
     _cogl_sub_texture_copy_region (data, full_data,
                                    0, 0,
                                    sub_tex->sub_x,
@@ -493,11 +479,11 @@ _cogl_sub_texture_get_data (CoglTexture     *tex,
                                    full_rowstride,
                                    bpp);
   else
-    byte_size = 0;
+    ret = FALSE;
 
   g_free (full_data);
 
-  return byte_size;
+  return ret;
 }
 
 static CoglPixelFormat
@@ -545,11 +531,12 @@ cogl_sub_texture_vtable =
     _cogl_sub_texture_transform_quad_coords_to_gl,
     _cogl_sub_texture_get_gl_texture,
     _cogl_sub_texture_set_filters,
-    _cogl_sub_texture_ensure_mipmaps,
+    _cogl_sub_texture_pre_paint,
     _cogl_sub_texture_ensure_non_quad_rendering,
-    _cogl_sub_texture_set_wrap_mode_parameter,
+    _cogl_sub_texture_set_wrap_mode_parameters,
     _cogl_sub_texture_get_format,
     _cogl_sub_texture_get_gl_format,
     _cogl_sub_texture_get_width,
-    _cogl_sub_texture_get_height
+    _cogl_sub_texture_get_height,
+    NULL /* is_foreign */
   };
