@@ -30,6 +30,7 @@
 #endif
 
 #include "cogl.h"
+#include "cogl-debug.h"
 #include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-texture-pixmap-x11.h"
@@ -42,6 +43,7 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-xlib.h"
+#include "cogl-material-opengl-private.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -886,12 +888,34 @@ _cogl_texture_pixmap_x11_free_glx_pixmap (CoglTexturePixmapX11 *tex_pixmap)
 {
   if (tex_pixmap->glx_pixmap)
     {
+      CoglXlibTrapState trap_state;
+
       _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
       if (tex_pixmap->pixmap_bound)
         glXReleaseTexImage (_cogl_xlib_get_display (), tex_pixmap->glx_pixmap,
                             GLX_FRONT_LEFT_EXT);
+
+      /* FIXME - we need to trap errors and synchronize here because
+       * of ordering issues between the XPixmap destruction and the
+       * GLXPixmap destruction.
+       *
+       * If the X pixmap is destroyed, the GLX pixmap is destroyed as
+       * well immediately, and thus, when Cogl calls glXDestroyPixmap()
+       * it'll cause a BadDrawable error.
+       *
+       * this is technically a bug in the X server, which should not
+       * destroy either pixmaps until the call to glXDestroyPixmap(); so
+       * at some point we should revisit this code and remove the
+       * trap+sync after verifying that the destruction is indeed safe.
+       *
+       * for reference, see:
+       *   http://bugzilla.clutter-project.org/show_bug.cgi?id=2324
+       */
+      _cogl_xlib_trap_errors (&trap_state);
       glXDestroyPixmap (_cogl_xlib_get_display (), tex_pixmap->glx_pixmap);
+      XSync (_cogl_xlib_get_display (), False);
+      _cogl_xlib_untrap_errors (&trap_state);
 
       tex_pixmap->glx_pixmap = None;
       tex_pixmap->pixmap_bound = FALSE;
@@ -1118,18 +1142,14 @@ _cogl_texture_pixmap_x11_set_region (CoglTexture     *tex,
                                      int              dst_y,
                                      unsigned int     dst_width,
                                      unsigned int     dst_height,
-                                     int              width,
-                                     int              height,
-                                     CoglPixelFormat  format,
-                                     unsigned int     rowstride,
-                                     const guint8    *data)
+                                     CoglBitmap      *bmp)
 {
   /* This doesn't make much sense for texture from pixmap so it's not
      supported */
   return FALSE;
 }
 
-static int
+static gboolean
 _cogl_texture_pixmap_x11_get_data (CoglTexture     *tex,
                                    CoglPixelFormat  format,
                                    unsigned int     rowstride,
@@ -1290,7 +1310,7 @@ static void
 _cogl_texture_pixmap_x11_set_wrap_mode_parameters (CoglTexture *tex,
                                                    GLenum       wrap_mode_s,
                                                    GLenum       wrap_mode_t,
-                                                   GLenum       wrap_mode_r)
+                                                   GLenum       wrap_mode_p)
 {
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
   CoglHandle child_tex;
@@ -1301,7 +1321,7 @@ _cogl_texture_pixmap_x11_set_wrap_mode_parameters (CoglTexture *tex,
   _cogl_texture_set_wrap_mode_parameters (child_tex,
                                           wrap_mode_s,
                                           wrap_mode_t,
-                                          wrap_mode_r);
+                                          wrap_mode_p);
 }
 
 static CoglPixelFormat
