@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __CLUTTER_BACKEND_X11_H__
@@ -30,6 +30,9 @@
 
 #include "clutter-x11.h"
 
+#include "clutter-keymap-x11.h"
+#include "xsettings/xsettings-client.h"
+
 G_BEGIN_DECLS
 
 #define CLUTTER_TYPE_BACKEND_X11                (clutter_backend_x11_get_type ())
@@ -82,8 +85,15 @@ struct _ClutterBackendX11
 
   Time last_event_time;
 
-  ClutterInputDevice *core_pointer;
-  ClutterInputDevice *core_keyboard;
+  ClutterDeviceManager *device_manager;
+
+  XSettingsClient *xsettings;
+  Window xsettings_xwin;
+
+  ClutterKeymapX11 *keymap;
+  int xkb_event_base;
+  gboolean use_xkb;
+  gboolean have_xkb_autorepeat;
 };
 
 struct _ClutterBackendX11Class
@@ -105,6 +115,9 @@ struct _ClutterBackendX11Class
                             XEvent            *xevent);
 };
 
+/* platform-specific event data */
+typedef struct _ClutterEventX11 ClutterEventX11;
+
 void   _clutter_backend_x11_events_init (ClutterBackend *backend);
 void   _clutter_backend_x11_events_uninit (ClutterBackend *backend);
 
@@ -142,6 +155,15 @@ _clutter_x11_get_device_for_xid (XID id);
 void
 _clutter_x11_select_events (Window xwin);
 
+ClutterEventX11 *
+_clutter_event_x11_new (void);
+
+ClutterEventX11 *
+_clutter_event_x11_copy (ClutterEventX11 *event_x11);
+
+void
+_clutter_event_x11_free (ClutterEventX11 *event_x11);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_BACKEND_X11_H__ */
