@@ -14,12 +14,17 @@ test_rotate_main (int argc, char *argv[])
   ClutterAlpha     *alpha;
   ClutterBehaviour *r_behave;
   ClutterActor     *stage;
-  ClutterActor     *hand, *label;
+  ClutterActor     *hand, *label, *rect;
   ClutterColor      stage_color = { 0xcc, 0xcc, 0xcc, 0xff };
+  ClutterColor      rect_color;
   gchar            *file;
 
   clutter_init (&argc, &argv);
 
+  /* Make a timeline */
+  timeline = clutter_timeline_new (7692);
+  clutter_timeline_set_loop (timeline, TRUE);
+
   stage = clutter_stage_get_default ();
 
   clutter_stage_set_color (CLUTTER_STAGE (stage),
@@ -33,9 +38,18 @@ test_rotate_main (int argc, char *argv[])
 
   g_free (file);
 
+  clutter_color_from_string (&rect_color, "#ce5c00ff");
+  rect = clutter_rectangle_new_with_color (&rect_color);
+  clutter_actor_add_effect_with_name (rect, "blur", clutter_blur_effect_new ());
+  clutter_actor_set_position (rect, 340, 140);
+  clutter_actor_set_size (rect, 150, 150);
+
   clutter_actor_set_position (hand, 240, 140);
-  clutter_actor_show (hand);
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage), hand);
+  clutter_actor_add_effect_with_name (hand, "desaturate", clutter_desaturate_effect_new (0.75));
+  clutter_actor_add_effect_with_name (hand, "blur", clutter_blur_effect_new ());
+  clutter_actor_animate_with_timeline (hand, CLUTTER_LINEAR, timeline,
+                                       "@effects.desaturate.factor", 1.0,
+                                       NULL);
 
   label = clutter_text_new_with_text ("Mono 16",
                                       "The Wonder\n"
@@ -44,13 +58,9 @@ test_rotate_main (int argc, char *argv[])
   clutter_text_set_line_alignment (CLUTTER_TEXT (label), PANGO_ALIGN_CENTER);
   clutter_actor_set_position (label, 150, 150);
   clutter_actor_set_size (label, 500, 100);
-  clutter_actor_show (label);
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage), label);
-  
-  /* Make a timeline */
-  timeline = clutter_timeline_new (7692); /* num frames, fps */
-  clutter_timeline_set_loop (timeline, TRUE);
 
+  clutter_container_add (CLUTTER_CONTAINER (stage), rect, hand, label, NULL);
+  
   /* Set an alpha func to power behaviour */
   alpha = clutter_alpha_new_full (timeline, CLUTTER_LINEAR);
 
@@ -66,6 +76,7 @@ test_rotate_main (int argc, char *argv[])
   /* Apply it to our actor */
   clutter_behaviour_apply (r_behave, hand);
   clutter_behaviour_apply (r_behave, label);
+  clutter_behaviour_apply (r_behave, rect);
 
   /* start the timeline and thus the animations */
   clutter_timeline_start (timeline);
